#include <iostream>
#include <stdexcept>

using namespace std;

//klasa kojom se opisuje lista
class Lista
{
public:
    //ugnjezdena klasa kojom se opisuje jedan element liste
    class Element
    {
    public:
        // metod koji vraca vrednost elementa
        int Vrednost() const
        {
            return _Vrednost;
        }

        //metod koji vraca pokazivac na sledeci element u listi
        Element* Sledeci() const
        {
            return _Sledeci;
        }

    private:
        //konstruktor klase Element
        Element( int v, Element* l )
            : _Vrednost(v),
              _Sledeci(l)
        {}

        //destruktor klase Element
        ~Element(){}

        //operator dodeljivanja
        Element& operator = ( const Element& );

        //kopi-konstruktor
        Element( const Element& l )
            : _Vrednost(l._Vrednost),
              _Sledeci(l._Sledeci)
        {}

        int _Vrednost;
        Element* _Sledeci;

        //definisanje klase Lista kao prijateljske klase
        friend class Lista;
    };

    //konstruktor klase Lista
    Lista()
        : _Pocetak(0)
    {}

    //kopi-konstriktor
    Lista( const Lista& l )
    {
        init(l);
    }

    //destruktor klase Lista
    ~Lista()
    {
        deinit();
    }

    //operator dodeljivanja
    Lista& operator = ( const Lista& l )
    {
        if( this != &l )
        {
            deinit();
            init(l);
        }

        return *this;
    }

    //metod za dodavanje novog elementa na pocetak liste
    void DodajNaPocetak( int n )
    {
        Element* novi = new Element( n, _Pocetak );
        _Pocetak = novi;
    }

    //metod za dodavanje novog elementa na kraj liste
    void DodajNaKraj( int n )
    {
        Element* novi = new Element( n, 0 );
        if( _Pocetak )
        {
            Element* p;
            for( p=_Pocetak; p->Sledeci(); p=p->Sledeci() )
                ;
            p->_Sledeci = novi;
        }
        else
            _Pocetak = novi;
    }

    //operator []
    int operator[]( int i ) const
    {
        return element(i);
    }

    int& operator[]( int i )
    {
        return element(i);
    }

    //metod koji vraca duzinu liste
    int Duzina() const
    {
        int n=0;
        for( Element* p=_Pocetak; p; p=p->Sledeci() )
            n++;

        return n;
    }

    // metod koji vraca adresu pocetka liste
    Element* Pocetak()
    {
        return _Pocetak;
    }

private:
    void init( const Lista& l )
    {
        if( l._Pocetak )
        {
            _Pocetak = new Element( *l._Pocetak );
            Element* p = _Pocetak;
            while(p->Sledeci())
            {
                p->_Sledeci = new Element( *p->Sledeci() );
                p = p->Sledeci();
            }
        }
        else
            _Pocetak = 0;
    }

    void deinit()
    {
        for( Element* p=_Pocetak; p; )
        {
            Element* t = p->Sledeci();
            delete p;
            p = t;
        }
    }

    //metod koji vraca vrednost i-tog elementa u listi (vazi samo ako postoji i+1-ti element)
    int& element(int i) const
    {
        if(i>=Duzina())
            throw out_of_range("Element sa zadatim indeksom ne postoji");

        Element* p = _Pocetak;
        for( int j=0; j<i; j++ )
            p = p->Sledeci();

        return p->_Vrednost;
    }

    Element* _Pocetak;
};

int main()
{

    try
    {
        cout<< "Lista l:" << endl;
        Lista l;

        for( int i=0; i<10; i++ )
            l.DodajNaPocetak(i);

        int n = l.Duzina();
        for( int i=0; i<n; i++ )
        cout << l[i] << ' ';
        cout << endl;

        cout<< "Lista l posle mnozenja vrednosti elemenata sa 2:" << endl;
        for( int i=0; i<10; i++ )
            l[i] *= 2;
        for( int i=0; i<n; i++ )
            cout << l[i] << ' ';
        cout << endl;

        cout << l[25] << endl;

        cout<< "Lista l:" << endl;

        for( Lista::Element* p=l.Pocetak(); p; p=p->Sledeci() )
            cout << p->Vrednost() << ' ';
        cout << endl;

    }
    catch(const out_of_range& exception)
    {
        cout <<  exception.what() << endl;
    }
    catch(...)
    {
        cout << "Uhvacen je izuzetak ..." << endl;
    }

    return 0;
}
